import sympy as sy
from vec2skew import vec2skew

# Define symbolic variables
l1, l2, l3 = sy.symbols('l1 l2 l3', real=True)
theta1, theta2, theta3 = sy.symbols('theta1 theta2 theta3', real=True)

# Axis vector
n = sy.Matrix([0, 0, 1])

# Identity matrix
R00 = sy.eye(3)

# Define rotation and transformation matrices
R01 = sy.Matrix([
    [sy.cos(theta1), -sy.sin(theta1), 0],
    [sy.sin(theta1), sy.cos(theta1), 0],
    [0, 0, 1]
    ])
O01 = sy.Matrix([0, 0, 0]) #use print(O01.shape), 3x1
H01 = sy.Matrix.vstack(
    sy.Matrix.hstack(R01, O01),  # Ensure O01 is a column vector
    sy.Matrix([[0, 0, 0, 1]])  # Add the last row as a 1x4 matrix
    )
#print(H01.shape)

R12 = sy.Matrix([
    [sy.cos(theta2), -sy.sin(theta2), 0],
    [sy.sin(theta2), sy.cos(theta2), 0],
    [0, 0, 1]])
O12 = sy.Matrix([l1, 0, 0])
H12 = sy.Matrix.vstack(
     sy.Matrix.hstack(R12, O12),  # Ensure O12 is a 3x1 column vector
    sy.Matrix([[0, 0, 0, 1]])  # Last row as a 1x4 matrix
    )
#print(H12.shape)

R23 = sy.Matrix([
    [sy.cos(theta3), -sy.sin(theta3), 0],
    [sy.sin(theta3), sy.cos(theta3), 0],
    [0, 0, 1]
])
O23 = sy.Matrix([l2, 0, 0])
H23 = sy.Matrix.vstack(
    sy.Matrix.hstack(R23, O23),  # Ensure O23 is a 3x1 column vector
    sy.Matrix([[0, 0, 0, 1]])  # Last row as a 1x4 matrix
)
#print(H23.shape)


# Compute intermediate transformation matrices
H02 = sy.simplify(H01 * H12)
H03 = sy.simplify(H02 * H23)

R02 = H02[:3, :3]
R03 = H03[:3, :3]

O02 = H02 * sy.Matrix([0, 0, 0, 1])
O03 = H03 * sy.Matrix([0, 0, 0, 1])
E0 = H03 * sy.Matrix([l3, 0, 0, 1])


o01 = sy.Matrix(O01)
o02 = sy.Matrix(O02[:3])
o03 = sy.Matrix(O03[:3])
e0 = sy.Matrix(E0[:3])



# a) J_E
Jv_E = sy.Matrix.hstack(
    vec2skew(R00 * n) * (e0 - o01),
    vec2skew(R01 * n) * (e0 - o02),
    vec2skew(R02 * n) * (e0 - o03)
)

#Jw_E = [R00*n, R01*n, R02*n];
Jw_E = sy.Matrix.hstack(R00 * n, R01 * n, R02 * n)

J_E = sy.Matrix.vstack(Jv_E, Jw_E)
#print(J_E)


# b) J_P
p0 = sy.Matrix(o02)
Jv_P = sy.Matrix.hstack(
    vec2skew(R00 * n) * (p0 - o01),
    sy.zeros(3, 1),
    sy.zeros(3, 1)
)
Jw_P = sy.Matrix.hstack(R00 * n, sy.zeros(3, 1), sy.zeros(3, 1))

J_P = sy.Matrix.vstack(Jv_P, Jw_P)

# Output the Jacobians
print("J_E:")
sy.pprint(J_E)

print("\nJ_P:")
sy.pprint(J_P)
